"use strict";exports.id=490,exports.ids=[490],exports.modules={6001(e,i,r){i.loadLibrary=function(e){let i=o[e];if("string"!=typeof i){let r;r="jquery"===e?s:(0,t.join)(d,e);try{i=(0,n.readFileSync)(r).toString()}catch(t){console.log(`Unable to load library ${e} at ${r}`),i=""}o[e]=i}return i};const t=r(6928),n=r(9896),o={},a="dist"===(0,t.basename)(__dirname)?(0,t.dirname)(__dirname):(0,t.dirname)((0,t.dirname)(__dirname)),d=(0,t.join)(a,"../../node_modules/typescript/lib"),s=(0,t.join)(a,"lib/jquery.d.ts")}};
//# sourceMappingURL=https://main.vscode-cdn.net/sourcemaps/7662837fe25ed7997bd036bd79e745dd27fc5597/extensions/html-language-features/server/dist/node/490.htmlServerMain.js.map