export class BaseSVGFactory {
    create(width: any, height: any, skipDimensions?: boolean): void;
    createElement(type: any): void;
    /**
     * @ignore
     */
    _createSVG(type: any): void;
}
export class DOMSVGFactory extends BaseSVGFactory {
    /**
     * @ignore
     */
    _createSVG(type: any): any;
}
